#===========================================================================
#  Filename: BulkImport.ksh
#    Author: Craig Schneider
# Orig Date: 08/11/15
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: BulkImport from a file to a database table.
#===========================================================================

. $EZ_SHARED_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: BulkImport -FileName FileName [-Quiet] [-Verbose]

      -FileName: The name of the file to bulk import.
         -Quiet: Suppress non critical messages about operations performed.
       -Verbose: Display verbose messages about operations being performed.

!
   exit 1
}

EZ_DRYRUN=FALSE
QUIET_MODE=FALSE
RECORD_SRC=
FILE_NAME=
VERBOSE_MODE=FALSE

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -filename)
          FILE_NAME="$2"
          shift
          shift ;;
      -quiet)
          QUIET_MODE=TRUE
          shift ;;
      -verbose)
          VERBOSE_MODE=TRUE
          shift ;;
      -?) Usage
          shift ;;
      *)  if [ "$1" != "${1#-}" ] ; then
             Usage "Invalid argument $1"
          else
             if [ "$TEST_DLL_LIST" = "" ] ; then
                TEST_DLL_LIST="$1"
             else
                TEST_DLL_LIST="$TEST_DLL_LIST,$1"
             fi
          fi
          shift ;;
   esac
done

if [ "$FILE_NAME" = "" ] ; then
   Usage "Must specify a FileName to import."
fi

BCP_EXE="C:/Program Files/Microsoft SQL Server/110/Tools/Binn/bcp.exe"

TABLE_NAME="${FILE_NAME%%.*}"
BCP_FORMAT_FILE_NAME="$FILE_NAME.fmt"

"$BCP_EXE" $TABLE_NAME in "$FILE_NAME" -f "$BCP_FORMAT_FILE_NAME" \
           -U NP_DBA -P DBA1A -S PCC-DEV5 -d Dev_NP_A | \
           egrep -v 'Starting copy...|rows sent to|rows copied.|Network packet size|Clock Time' | \
           sed -e '/^$/d'
EXIT_CODE=$?
if [ "$EXIT_CODE" -ne 0 ] ; then
   exit $EXIT_CODE
fi

if [ "$QUIET_MODE" != TRUE ] ; then
   ROWS_IMPORTED=`wc -l $FILE_NAME | awk '{print $1}'`
   echo "Successfully imported $ROWS_IMPORTED rows into $TABLE_NAME."
fi

exit $EXIT_CODE
