#===========================================================================
# FileName: <<FileName>>
#   Author: Craig Schneider
#  Created: 2017-05-16
# Synopsis: Reset the feed files for the specified feed name.
# <<Copyright>>
#===========================================================================

FEED_NAME="CATS"
RESET_ARCHIVE=FALSE
RESET_DUP=FALSE
RESET_SOURCE_ARCHIVE=FALSE
IN_FILE_TRK_DELETE=FALSE
TRUNCATE_NSM=FALSE

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: FeedFilesReset -Feed <FeedName> [-DeleteTracking] [-ResetAll] 
               [-ResetArchive] [-ResetDup] [-ResetSourceArchive] [-TruncateNSM]

    -DeleteTracking: Delete ALL IN_FILE_TRK records for the feed.
              -Feed: The name of the feed to reset files for (default $FEED_NAME).
          -ResetAll: Specifies that everything is reset/deleted/truncated.
      -ResetArchive: Specifies that the local Archive files are reset.
          -ResetDup: Specifies that the Error/DuplicateArchived files are reset.
-ResetSourceArchive: Specifies that the Source Archive files are reset.
       -TruncateNSM: Truncate the NSM table for the feed.
!
   exit 1
}

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
    -deletetracking) IN_FILE_TRK_DELETE=TRUE
                     shift ;;
              -feed) FEED_NAME="$2"
                     shift
                     shift ;;
      -resetarchive) RESET_ARCHIVE=TRUE
                     shift ;;
          -resetall) IN_FILE_TRK_DELETE=TRUE
                     RESET_ARCHIVE=TRUE
                     RESET_DUP=TRUE
                     RESET_SOURCE_ARCHIVE=TRUE
                     TRUNCATE_NSM=TRUE
                     shift ;;
          -resetdup) RESET_DUP=TRUE
                     shift ;;
-resetsourcearchive) RESET_SOURCE_ARCHIVE=TRUE
                     shift ;;
       -truncatensm) TRUNCATE_NSM=TRUE
                     shift ;;
                  *) if [ "$1" != "-?" ] ; then
                        Usage "Invalid argument: $1"
                     else
                        Usage
                     fi ;;
   esac
done

GLOBAL_FILES_DIR="C:/GlobalFiles"
LOCAL_FILES_DIR="C:/LocalFiles"
ERROR_FEED_DIR="$GLOBAL_FILES_DIR/InputDataFeedErrors/$FEED_NAME/Error"
INBOUND_FEED_DIR="$GLOBAL_FILES_DIR/LandingZone/Inbound/$FEED_NAME"
INBOUND_FEED_RESET_DIR="$INBOUND_FEED_DIR.RESET"
LOCAL_FEED_DIR="$LOCAL_FILES_DIR/InputDataFeeds/$FEED_NAME"
SOURCE_ARCHIVE_DIR="$INBOUND_FEED_DIR/Archive"
ARCHIVE_FEED_DIR="$LOCAL_FILES_DIR/InputDataFeeds/Archive/$FEED_NAME"

if [ "$RESET_ARCHIVE" = FALSE ] ; then
   ARCHIVE_FEED_DIR=
fi
if [ "$RESET_SOURCE_ARCHIVE" = FALSE ] ; then
   SOURCE_ARCHIVE_DIR=
fi

for DIRECTORY in $ARCHIVE_FEED_DIR         \
                 $SOURCE_ARCHIVE_DIR       \
                 $LOCAL_FEED_DIR/Input     \
                 $LOCAL_FEED_DIR/Parse     \
                 $LOCAL_FEED_DIR/Load
do
   if [ -d "$DIRECTORY" ] ; then
      rm -rf "$DIRECTORY"
      echo "Successfully removed dir $DIRECTORY."
   fi
done

if [ -d "$ERROR_FEED_DIR" ] ; then
   for FILE_OR_DIR in `ls $ERROR_FEED_DIR`
   do
       if [ -f "$ERROR_FEED_DIR/$FILE_OR_DIR" ] ; then
          rm -f $ERROR_FEED_DIR/$FILE_OR_DIR
          echo "Successfully removed $ERROR_FEED_DIR/$FILE_OR_DIR."
       else
          if [ "$FILE_OR_DIR" != DuplicateArchived -o "$RESET_DUP" = TRUE ] ; then
             rm -rf $ERROR_FEED_DIR/$FILE_OR_DIR
             echo "Successfully removed dir $ERROR_FEED_DIR/$FILE_OR_DIR."
          fi
       fi
   done
   #rm -rf "$DIRECTORY"
   #echo "Successfully removed dir $DIRECTORY."
fi

for DIRECTORY in $INBOUND_FEED_DIR
do
   if [ -d "$DIRECTORY" ] ; then
      for FILE_OR_DIR in `ls $DIRECTORY`
      do
          if [ -f "$DIRECTORY/$FILE_OR_DIR" ] ; then
             rm -f $DIRECTORY/$FILE_OR_DIR
             echo "Successfully removed $DIRECTORY/$FILE_OR_DIR."
          fi
      done
   fi
done

if [ -d "$INBOUND_FEED_RESET_DIR" ] ; then
   for FILE_OR_DIR in `ls $INBOUND_FEED_RESET_DIR`
   do
       if [ -f "$INBOUND_FEED_RESET_DIR/$FILE_OR_DIR" ] ; then
          cp -p $INBOUND_FEED_RESET_DIR/$FILE_OR_DIR \
                      $INBOUND_FEED_DIR/$FILE_OR_DIR
          echo "Successfully created $INBOUND_FEED_DIR/$FILE_OR_DIR."
       fi
   done
fi

if [ "$IN_FILE_TRK_DELETE" = TRUE ] ; then
   C4DB.exe -c "delete from IN_FILE_TRK where INPUT_DATA_FEED_NAME = '$FEED_NAME'"
fi

if [ "$TRUNCATE_NSM" = TRUE ] ; then
   #C4DB.exe -variables -set Feedback False \
   #         -c "select LOAD_CONNECTION_NAME, LOAD_NSM_TABLE_NAME
   #             from INPUT_DATA_FEED
   #             where INPUT_DATA_FEED_NAME = '$FEED_NAME'"
   # using a delete so the number of rows deleted is reported.
   C4DB.exe -s SS_DevX_WH_SS -c "delete from ${FEED_NAME}_NSM"
fi

echo
echo "============================================================================"
FeedFilesList.sh -Feed $FEED_NAME
