#===========================================================================
#  Filename: InfragisticsSetup.ksh
#    Author: Craig Schneider
# Orig Date: 11/20/12
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Setup the Infragistics dlls directory.
#===========================================================================

. $EZ_SHARED_ENV

PREV_VER=15.1
CURR_VER=16.2

PREV_3RD_DIR="C:/3rdParty/Infragistics${PREV_VER}"
CURR_3RD_DIR="C:/3rdParty/Infragistics${CURR_VER}"

if [ ! -d "$PREV_3RD_DIR" ] ; then
   echo "ERROR: Missing Previous 3rdParty Directory $PREV_3RD_DIR."
   exit 1
fi
if [ ! -d "$CURR_3RD_DIR" ] ; then
   echo    "Warning: Missing Current 3rdParty Directory $CURR_3RD_DIR."
   echo -n "Warning: create it <y,n>? "
   read ANSWER
   if [ "$ANSWER" != y ] ; then
      exit 1
   fi
   mkdir -p $CURR_3RD_DIR
fi

INFRA_ROOT="C:/Program Files (x86)/Infragistics"
NETADV_ROOT="$INFRA_ROOT/20${CURR_VER}"

for PREV_FILE in `( cd "$PREV_3RD_DIR" ; ls )`
do
   PREV_3RD_DIR_IF_ALLOWED="C:/NoSuchDir"
   CURR_FILE=`echo "$PREV_FILE" | \
                   sed -e "s/v$PREV_VER/v$CURR_VER/" \
                       -e "s/Infragistics4.Web/Infragistics45.Web/"`
   if [ "$CURR_FILE" = "$PREV_FILE" ] ; then
      #TMP_FILE_2="${CURR_FILE#Infragistics4.Web}"
      #if [ "$CURR_FILE" != "$TMP_FILE_1" -a \
      #     "$CURR_FILE" != "$TMP_FILE_2" ] ; then
      #   PREV_3RD_DIR_IF_ALLOWED="$PREV_3RD_DIR"
      #else
         echo "ERROR: Failed to determine current file name for $PREV_FILE."
         exit 1
      #fi
   fi
   NEW_FILE_FULL_NAME=
   for SEARCH_DIR in "$NETADV_ROOT/Windows Forms/CLR4.0/Bin" \
                     "$NETADV_ROOT/ASP.NET/CLR4.5/Bin" \
                     "$PREV_3RD_DIR_IF_ALLOWED"
   do
      NEW_FILE_FULL_NAME="$SEARCH_DIR/$CURR_FILE"
      if [ -f "$NEW_FILE_FULL_NAME" ] ; then
         break
      #else
      #   echo "MISSING: $NEW_FILE_FULL_NAME"
      fi 
   done
   if [ -f "$NEW_FILE_FULL_NAME" ] ; then
      CURR_FILE_FULL_NAME="$CURR_3RD_DIR/$CURR_FILE"
      if [ -f "$CURR_FILE_FULL_NAME" ] ; then
         echo "Successfully found existing file $CURR_FILE_FULL_NAME."
      else
         cp "$NEW_FILE_FULL_NAME" "$CURR_FILE_FULL_NAME"
         if [ $? -eq 0 ] ; then
            echo "UPDATED: $CURR_FILE"
         else
            echo "ERROR: Failed creating file $CURR_FILE_FULL_NAME."
            exit 1
         fi
      fi
   else
      echo "ERROR: Failed to find a corresponding current file."
      echo "ERROR: Prev: $PREV_FILE"
      echo "ERROR: Curr: $CURR_FILE"
   fi
done

exit 0
