#===========================================================================
#  Filename: Shields.ksh
#    Author: Craig Schneider
# Orig Date: 02/22/2016
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Get the version of a file.
#===========================================================================

. $EZ_SHARED_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo "$1"
   fi
   cat <<!

Usage: Shields [-Down] [-Up]

      -Down: Bring the shields down, open up blocked ports.
        -Up: Bring the shields up, block ports.
!
   exit 1
}

UP_DOWN_MODE=UP

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -down) UP_DOWN_MODE=DOWN
             shift ;;
        -up) UP_DOWN_MODE=UP
             shift ;;
          *) Usage
             shift ;;
   esac
done

OUT_FILE_NAME=$TMPDIR/shields.out
for RULE_AND_PORT in FTP:21 Greenplum:5432 Hive:10000 MySQL:3306 Netezza:5480 \
                     Oracle:1521 Redshift:5439 SFTP:22 SQLServer:1433 \
                     Teradata:1025
do
   RULE_NAME="__${RULE_AND_PORT%:*}"
   TCP_PORT="${RULE_AND_PORT#*:}"
   if [ "$UP_DOWN_MODE" = DOWN ] ; then
      netsh advfirewall firewall delete rule name="$RULE_NAME" dir=out \
            protocol=TCP remoteport=$TCP_PORT > "$OUT_FILE_NAME"
      if [ $? -eq 0 ] ; then
         echo "Successfully brought the shields down deleted rule $RULE_NAME."
      else
         egrep -qs "No rules match the specified criteria." "$OUT_FILE_NAME"
         if [ $? -eq 0 ] ; then
            echo "Successfully detected the shields are already down for rule $RULE_NAME."
         else
            echo "EXIT_CODE=$?"
            echo
            echo "ERROR: Failed to bring the shields down for rule $RULE_NAME."
            echo -n "ERROR: "
            cat "$OUT_FILE_NAME"
            exit 1
         fi
      fi
   else
      netsh advfirewall firewall add rule name="$RULE_NAME" dir=out \
            protocol=TCP remoteport=$TCP_PORT action=block > \
            "$OUT_FILE_NAME"
      if [ $? -eq 0 ] ; then
         echo "Successfully brought the shields up added rule $RULE_NAME."
      else
         echo
         echo "ERROR: Failed to bring the shields up for rule $RULE_NAME."
         echo -n "ERROR: "
         cat "$OUT_FILE_NAME"
         exit 1
      fi
   fi
done
rm -f "$OUT_FILE_NAME"

if [ "$UP_DOWN_MODE" = UP ] ; then
   netsh advfirewall show privateprofile state | egrep -qs 'ON'
   if [ $? -ne 0 ] ; then
      echo
      echo "ERROR: The Windows Firewall Private Profile is disabled."
      echo "ERROR: To enable it, use the following command:"
      echo
      echo "       netsh advfirewall set privateprofile state on"
      echo
      exit 1
   fi
fi

exit 0
