#===========================================================================
#  Filename: SyncTbl.ksh
#    Author: Craig Schneider
# Orig Date: 02/26/16
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Syncronize tables to .tbl files.
#===========================================================================

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: $0 [-no] FileName(s)...

      -no: All user questions will be implicitly answered with a no.

!
   exit 1
}

STABALIZE_ARGS=
FILE_LIST=
typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
     -no) STABALIZE_ARGS="$STABALIZE_ARGS -no"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) FILE_LIST="$FILE_LIST $1"
          shift ;;
    esac
done

FILE_COUNT=0
UNSTABLE_COUNT=0

typeset -u TABLE_NAME
for FILE_NAME in $FILE_LIST
do
   TABLE_NAME="${FILE_NAME%.tbl.4db}"

   COMMENTS=`egrep '^\-\- ' $FILE_NAME`

   desc.sh -create $TABLE_NAME > $FILE_NAME.fix
   if [ "$COMMENTS" != "" ] ; then
      egrep '^include' $FILE_NAME.fix      > Comments.fix 
      echo                                >> Comments.fix
      echo "$COMMENTS"                    >> Comments.fix
      sed -e '/include /d' $FILE_NAME.fix >> Comments.fix
      mv Comments.fix $FILE_NAME.fix
   fi
   dos2unix.sh -r $FILE_NAME.fix
   src.ksh stabalize $STABALIZE_ARGS -diffs -r $FILE_NAME.fix $FILE_NAME
   if [ $? -ne 0 ] ; then
      let UNSTABLE_COUNT=UNSTABLE_COUNT+1
   fi
   let FILE_COUNT=FILE_COUNT+1
done

if [ $UNSTABLE_COUNT -gt 0 ] ; then
   echo
   echo
   echo "Warning: Found $UNSTABLE_COUNT of $FILE_COUNT files to be out of compliance."
fi
