#===========================================================================
#  FileName: UpdateEZC4Packages.ksh
#    Author: Craig Schneider
# Orig Date: 11/30/16
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Korn Shell make system.
#===========================================================================

NPM_CMD=npm.cmd
WHENCE_NPM=`whence $NPM_CMD`
if [ "$WHENCE_NPM" = "" ] ; then
   echo "Warning: Detected $NPM_CMD is not installed."   
   exit 1
fi

PACKAGE_VERSION=`$NPM_CMD view ez-c4-packages@2.10 version --no-progress | \
                 tail -1 | cut -f1 -d ' ' | sed -e 's/[ -z]*@//'`
PACKAGE_VERSION_DIR=`echo "${PACKAGE_VERSION#0.}" | sed -e 's/-/./g'`
PACKAGE_FILE_NAME=package.c4-packages.json
PACKAGE_3RD_DIR="$EZ_3RDPACKAGES_ROOT"
PACKAGE_BASE_DIR="EZC4Packages$PACKAGE_VERSION_DIR"
PACKAGE_FULL_DIR="$PACKAGE_3RD_DIR/$PACKAGE_BASE_DIR"
PACKAGE_FILE_FULL_NAME="$PACKAGE_FULL_DIR/package.json"
PACKAGE_VERSION_FILE="$PACKAGE_FULL_DIR/version.txt"
PACKAGE_VERSION_INSTALLED=

if [ "$PACKAGE_VERSION" = "" ] ; then
   echo "ERROR: Failed determining the latest package version."
   echo "ERROR: Using command $NPM_CMD view ez-c4-packages@2.10 version --no-progress"
   exit 1
fi

if [ ! -d "$PACKAGE_3RD_DIR" ] ; then
   mkdir $PACKAGE_3RD_DIR
   echo "Successfully created directory $PACKAGE_3RD_DIR."
   touch $PACKAGE_3RD_DIR/NoNodePackages
   echo "Successfully created file $PACKAGE_3RD_DIR/NoNodePackages."
fi
if [ ! -d "$PACKAGE_3RD_DIR" ] ; then
   echo "Warning: Directory $PACKAGE_3RD_DIR does not exist."
   exit 0
fi

if [ ! -d "$PACKAGE_FULL_DIR" ] ; then
   mkdir "$PACKAGE_FULL_DIR"
   echo "Successfully created directory $PACKAGE_FULL_DIR."
fi

if [ -f "$PACKAGE_VERSION_FILE" ] ; then
   PACKAGE_VERSION_INSTALLED=`cat "$PACKAGE_VERSION_FILE"`
fi

if [ "$PACKAGE_VERSION" = "$PACKAGE_VERSION_INSTALLED" ] ; then
   echo "Successfully detected $PACKAGE_FULL_DIR is up to date ($PACKAGE_VERSION)."
else

   cat > $PACKAGE_FILE_FULL_NAME <<!
{
  "name": "update-ez-c4-packages",
  "version": "$PACKAGE_VERSION",
  "description": "Update ez-c4-packages Package from NPM.",
  "repository": "bitbucket:example/repo",
  "dependencies": {
    "ez-c4-packages": "$PACKAGE_VERSION"
  },
  "license": "MIT"
}
!
   chmod 777 "$PACKAGE_FILE_FULL_NAME"

   SAVE_PWD="$PWD"
   if [ "$PACKAGE_VERSION_INSTALLED" != "" ] ; then
      echo "Successfully updating $PACKAGE_FULL_DIR (from $PACKAGE_VERSION_INSTALLED)."
   else
      echo "Successfully updating $PACKAGE_FULL_DIR."
   fi
   PACKAGE_FULL_DIR_KSH=`echo "$PACKAGE_FULL_DIR" | \
                              /bin/sed 's{\([A-z]\):{/cygdrive/\l\1{'`
   cd "$PACKAGE_FULL_DIR_KSH"
   $NPM_CMD update --no-package-lock --no-progress | tr -d '\r' | \
                   sed -e '/added 1 package in /d'
   UPDATE_EXIT_CODE=$?
   cd "$SAVE_PWD"

   if [ "$UPDATE_EXIT_CODE" -eq 0 ] ; then
      cd "$PACKAGE_FULL_DIR_KSH"
      EXTRACT_OUTPUT_FILE="$TMPDIR/extract.$$"
      7z x -bd -pC4Rocks $PACKAGE_FULL_DIR/node_modules/ez-c4-packages/EZC4Packages.7z \
                          > "$EXTRACT_OUTPUT_FILE"
      if [ $? -ne 0 ] ; then
         cat "$EXTRACT_OUTPUT_FILE"
         rm -f "$EXTRACT_OUTPUT_FILE"
         echo "ERROR: Failed setting up directory $PACKAGE_FULL_DIR."
         exit 1
      fi
      rm -f "$EXTRACT_OUTPUT_FILE"
   
      cd "$SAVE_PWD"
      echo "Successfully updated $PACKAGE_FULL_DIR (to $PACKAGE_VERSION)."
      echo "$PACKAGE_VERSION" > "$PACKAGE_VERSION_FILE"
   else
      echo "ERROR: Failed updating $PACKAGE_FULL_DIR."
      exit 1
   fi
fi

# Purge old directories.
SAVE_DIR="$PWD"
PACKAGE_3RD_DIR_KSH=`echo "$PACKAGE_3RD_DIR" | \
                           /bin/sed 's{\([A-z]\):{/cygdrive/\l\1{'`
cd $PACKAGE_3RD_DIR_KSH
PACKAGE_MAJOR_MINOR="${PACKAGE_BASE_DIR%.*}"
LAST_DIR_0=`last_name.ksh -p .    ${PACKAGE_MAJOR_MINOR}*`
LAST_DIR_1=`last_name.ksh -p . -1 ${PACKAGE_MAJOR_MINOR}*`
LAST_DIR_2=`last_name.ksh -p . -2 ${PACKAGE_MAJOR_MINOR}*`
for PURGE_DIR in `ls -1d ${PACKAGE_MAJOR_MINOR}* | \
      egrep -v "$LAST_DIR_0|$LAST_DIR_1|$LAST_DIR_2"`
do
   if [ -d "$PURGE_DIR" ] ; then
      echo "Successfully removing old package $PURGE_DIR."
      rm -rf $PURGE_DIR
   fi
done
cd "$SAVE_PWD"

exit 0
