#===========================================================================
#  FileName: UpdateNodePackages.ksh
#    Author: Craig Schneider
# Orig Date: 10/19/16
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Korn Shell make system.
#===========================================================================

. $EZ_SHARED_ENV

if [ "$EZ_CANDIDATE_BUILD" = TRUE ] ; then
   echo "Successfully skipping UpdateNodePackages during candidate build."
   exit 0
fi

NPM_CMD=npm.cmd
WHENCE_NPM=`whence $NPM_CMD`
if [ "$WHENCE_NPM" = "" ] ; then
   echo "Warning: Detected $NPM_CMD is not installed."   
   exit 0
fi

PACKAGE_VERSION="2.10.13"
PACKAGE_FILE_NAME=package.nodes.json
PACKAGE_3RD_DIR="$EZ_3RDPACKAGES_ROOT"
#PACKAGE_VERSION=`grep '"version":' $PACKAGE_FILE_NAME | cut -f4 -d\"`
PACKAGE_BASE_DIR=NodePackages$PACKAGE_VERSION
PACKAGE_FULL_DIR="$PACKAGE_3RD_DIR/$PACKAGE_BASE_DIR"
PACKAGE_FILE_FULL_NAME="$PACKAGE_FULL_DIR/package.json"
INSTALLED_FILE_FULL_NAME="$PACKAGE_FULL_DIR/installed.json"
PERFORM_OUTDATED=TRUE
PERFORM_UPDATE=TRUE
EXIT_CODE=0

if [ ! -d "$PACKAGE_3RD_DIR" ] ; then
   echo "Warning: Directory $PACKAGE_3RD_DIR does not exist."
   exit 0
fi


if [ ! -d "$PACKAGE_FULL_DIR" ] ; then
   if [ "$1" = "-NoCreate" ] ; then
      EXISTING_NODE_DIRS=`echo $PACKAGE_3RD_DIR/NodePackages*`
      if [ "$EXISTING_NODE_DIRS" = "$PACKAGE_3RD_DIR/NodePackages*" ] ; then
         #echo "Warning: Not creating $PACKAGE_FULL_DIR since -NoCreate mode."
         exit 0
      fi 
   fi
   mkdir "$PACKAGE_FULL_DIR"
   echo "Successfully created directory $PACKAGE_FULL_DIR."
fi

cat > $PACKAGE_FILE_NAME <<!
{
  "name": "update-node-packages",
  "version": "$PACKAGE_VERSION",
  "description": "Update Node Packages From NPM.",
  "repository": "bitbucket:example/repo",
  "main": "index.js",
  "dependencies": {
    "angular-cli": "1.0.0-beta.28.3",
    "codelyzer": "3.2.2",
    "concurrently": "3.5.0",
    "tslint": "5.8.0",
    "typescript": "2.5.3",
    "typings": "2.1.1",
    "qunitjs": "2.4.1"
  },
  "keywords": [],
  "author": "",
  "license": "ISC"
}
!
chmod 777 "$PACKAGE_FILE_NAME"

SAVE_PWD="$PWD"
PACKAGE_FULL_DIR_KSH=`EZPath2Ksh "$PACKAGE_FULL_DIR"`
if [ -f "$INSTALLED_FILE_FULL_NAME" ] ; then
   cmp -s "$PACKAGE_FILE_NAME" "$INSTALLED_FILE_FULL_NAME"
   if [ "$?" -eq 0 ] ; then
      PERFORM_UPDATE=FALSE
   fi
fi

if [ "$PERFORM_UPDATE" = TRUE ] ; then
   cp "$PACKAGE_FILE_NAME" "$PACKAGE_FILE_FULL_NAME"
   chmod 777 "$PACKAGE_FILE_FULL_NAME"

   echo "Successfully updating $PACKAGE_FULL_DIR."
   cd "$PACKAGE_FULL_DIR_KSH"
   $NPM_CMD update --no-package-lock --no-progress | tr -d '\r' | \
                   sed -e '/ package in /d' -e '/ packages in /d'
   UPDATE_EXIT_CODE=$?
   cd "$SAVE_PWD"

   cp "$PACKAGE_FILE_NAME" "$INSTALLED_FILE_FULL_NAME"
   chmod 777 "$INSTALLED_FILE_FULL_NAME"
   rm -f "$PACKAGE_FILE_NAME"
   if [ "$UPDATE_EXIT_CODE" -eq 0 ] ; then
      echo "Successfully updated $PACKAGE_FULL_DIR."
      EXIT_CODE=0
   else
      echo "ERROR: Failed updating $PACKAGE_FULL_DIR."
      EXIT_CODE=1
   fi
fi

if [ "$PERFORM_OUTDATED" = TRUE ] ; then
   echo "Successfully checking for outdated node modules."
   cd "$PACKAGE_FULL_DIR_KSH"
   OUTDATED_FILE="$TEMP_DIR/npm_outdated.$$"
   $NPM_CMD outdated --no-progress > "$OUTDATED_FILE"
   cd "$SAVE_PWD"
   OUTDATED_RESULTS=`cat "$OUTDATED_FILE"`
   if [ "$OUTDATED_RESULTS" = "" ] ; then
      echo "Successfully detected $PACKAGE_FULL_DIR is up to date."
   else
      echo
      echo
      echo "Warning: Detected $PACKAGE_FULL_DIR is out of date."

      echo "$OUTDATED_RESULTS"
      echo
   fi
   rm -f "$OUTDATED_FILE"
   rm -f "$PACKAGE_FILE_NAME"
fi

for PACKAGE_OLD_BASE_DIR in NodePackages2.9.10
do
   PACKAGE_OLD_FULL_DIR="$PACKAGE_3RD_DIR/$PACKAGE_OLD_BASE_DIR"
   if [ -d "$PACKAGE_OLD_FULL_DIR" ] ; then
      echo "Successfully removing $PACKAGE_OLD_FULL_DIR."
      rm -rf "$PACKAGE_OLD_FULL_DIR"
      echo "Successfully removed  $PACKAGE_OLD_FULL_DIR."
   fi
done

exit $EXIT_CODE
