#===========================================================================
#  Filename: fam_destroy.ksh
#    Author: Craig Schneider
# Orig Date: 09/17/08
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Destroy an existing source control family.
#===========================================================================

. $EZ_SHARED_ENV

if [ "$1" = -? ] ; then
   cat <<!

Usage: fam destroy [FamilyName]

!
   exit 1
fi

if [ "$1" != "" ] ; then
   FAMILY_NAME="$1"
   shift
fi

if [ "$FAMILY_NAME" = "" ] ; then
   echo
   echo -n "Please specify the name of the source control family to destroy: "
   read FAMILY_NAME
fi

echo
echo -n "Are you sure you want to completely destroy $FAMILY_NAME <y,n>? "
read ANSWER
echo

if [ "$ANSWER" != y ] ; then
   echo "Warning: aborted by user."
   exit 1
fi

TOP_DIR="${EZ_FAMILYTREE_ROOT%/*}"  # can't use exported alias top
cd "$TOP_DIR"

svn update $FAMILY_NAME > $EZ_DEV_NULL

DESTROY_OUTPUT="$TMPDIR/fam_dest.$$"
src.ksh destroy $FAMILY_NAME 2>1 > "$DESTROY_OUTPUT"
egrep $EZ_EGREP_QUIET ' cannot be deleted| does not exist' "$DESTROY_OUTPUT"
EGREP_RET=$?
if [ "$EGREP_RET" -eq 0 ] ; then
   echo "Warning: Family $FAMILY_NAME does not exist."
   rm -f "$DESTROY_OUTPUT"
else
   cat "$DESTROY_OUTPUT"
   rm -f "$DESTROY_OUTPUT"
fi

FAMILY_ORDER="$EZ_FAMILYTREE_ROOT/Config/FamilyOrder.dat"
if [ -f "$FAMILY_ORDER" ] ; then
   sed -e "/^$FAMILY_NAME\$/d" \
       $FAMILY_ORDER > $FAMILY_ORDER.fix
   src.ksh stabalize -quiet -yes -r $FAMILY_ORDER.fix $FAMILY_ORDER
else
   echo "ERROR: Failed to find file $FAMILY_ORDER."
fi

fam.sh unsubscribe -family $FAMILY_NAME -quiet

echo
echo "Successfully destroyed family $FAMILY_NAME."
echo
echo "     Remember to:    start a new ksh or execute the dot command"
echo
exit 0
