#===========================================================================
#  Filename: fam_list.ksh
#    Author: Craig Schneider
# Orig Date: 07/10/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: List the available source control families.
#===========================================================================

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: fam list [-dryrun] [-no_subscribed]

       -dryrun: Display the commands to be executed, without executing them.
-no_subscribed: Do not display the subscribed status for each family.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi


EZ_DRYRUN=FALSE
DISPLAY_SUBSCRIBED=TRUE

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
        -dryrun) EZ_DRYRUN=TRUE
                 shift ;;
 -no_subscribed) DISPLAY_SUBSCRIBED=FALSE
                 shift ;;
              *) if [ "$1" != "-?" ] ; then
                    Usage "Invalid argument: $1"
                 else
                    Usage
                 fi ;;
    esac
done

#===========================================================================
# Perform the requested operations.
#===========================================================================

typeset -L20 FAMILY_NAME_FMT
for FAMILY_NAME in `svn list $EZ_SOURCE_CONTROL_ROOT/Source | \
                    tr -d '\r' | sed 's{/{{'`
do
   FAMILY_NAME_FMT="$FAMILY_NAME"
   DISPLAY_LINE="$FAMILY_NAME_FMT"
   if [ "$DISPLAY_SUBSCRIBED" = TRUE ] ; then
      fam.sh feature -name Subscribed -status -quiet -family "$FAMILY_NAME"
      if [ $? -eq 0 ] ; then
         DISPLAY_LINE="$DISPLAY_LINE SUBSCRIBED"
      fi
   fi
   echo "$DISPLAY_LINE                                "
done

exit 0
