#===========================================================================
# FileName: <<FileName>>
#   Author: Craig Schneider
#  Created: 2004-08-16
# Synopsis: Provide help on the suite of DevTools commands.
# <<Copyright>>
#===========================================================================

. $EZ_SHARED_ENV

HELP_FILE_NAME=`whence help.sh`
HELP_SCRIPTS_DIR="${HELP_FILE_NAME%help.sh}"
HELP_SUBSYSTEM=
HELP_COMMAND_NAME=
HELP_FORMAT=synopsis
HELP_OUTPUT_FORMAT=synopsis
HELP_EXCLUDED=
HELP_SYNOPSIS_FILE="$TMP/help.synopsis.$$"

EGREP_OPTS=
if [ "$EZ_OS_TYPE" = windows ] ; then
   EGREP_OPTS=-s
fi

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   cat <<!

Usage: help [CommandName] [-d Directory] [-s SubSystem] [-?] [-??] [-excluded]

        -??: Display this usage message.
         -?: Display usage help on all commands in the specified directory.
CommandName: A specific command to get help on.
         -d: Directory containing commands to generate help on.
  -excluded: Print only the names of commands being excluded.
         -s: SubSystem name (prefix of related commands to get help on).

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
function EZHelpFormatSynopsis
{
   HELP_COMMAND="$1"
   HELP_SYNOPSIS="$2"

   if [ "$HELP_SUBSYSTEM" != "" ] ; then
      HELP_COMMAND="$HELP_SUBSYSTEM ${HELP_COMMAND#$HELP_LS_QUALIFIER}"
   fi

   HELP_COMMAND=${HELP_COMMAND%.ksh}
   HELP_COMMAND=${HELP_COMMAND%.sh}
   HELP_COMMAND=${HELP_COMMAND%.vbs}

   typeset -L16 HELP_CMD_NAME
   HELP_CMD_NAME="$HELP_COMMAND"

   echo "$HELP_CMD_NAME $HELP_SYNOPSIS"
}

#===========================================================================
# Determine if the specified file should be excluded.
#===========================================================================
function EZHelpSetupExclusionList
{
   EXCLUSION_HELP_FORMAT="$1"

   HELP_EXCLUSION_LIST="|"
   for EXCLUDE_FILE in `grep -ls "^#  HelpType: Excluded" *`
   do
      HELP_EXCLUSION_LIST="${HELP_EXCLUSION_LIST}$EXCLUDE_FILE|"
   done

   if [ -f help.$EXCLUSION_HELP_FORMAT.exclude ] ; then
      for EXCLUDE_FILE in `cat help.$EXCLUSION_HELP_FORMAT.exclude`
      do
         HELP_EXCLUSION_LIST="${HELP_EXCLUSION_LIST}$EXCLUDE_FILE|"
      done
   fi
}

#===========================================================================
# Determine if the specified file should be excluded.
#===========================================================================
function EZHelpIsNotExcluded
{
   HELP_FILE="$1"
   
   PIPE_HELP_FILE="|$HELP_FILE|"
   if [ "${HELP_EXCLUSION_LIST#*$PIPE_HELP_FILE}" != \
        "$HELP_EXCLUSION_LIST" ] ; then
      if [ "$HELP_EXCLUDED" = -excluded ] ; then
         echo "Excluded: $HELP_FILE"
      fi
      return 1
   fi
   if [ "$HELP_EXCLUDED" = -excluded ] ; then
      return 1
   fi
   return 0
}

#===========================================================================
# Create the 1 line synopsis help list.
#===========================================================================
function EZHelpCreateSynopsisList
{
   if [ "$HELP_SUBSYSTEM" != "" ] ; then
      EZHelpFormatSynopsis "" \
       "Display the list of available $HELP_SUBSYSTEM commands with a synopsis."
      EZHelpFormatSynopsis "-?" \
       "Display the usage for each $HELP_SUBSYSTEM command."
   fi

   SUBSYSTEM_LIST=`grep -ls "^#  HelpType: SubSystemRoot" ${HELP_LS_QUALIFIER}*`
   for HELP_FILE in $SUBSYSTEM_LIST
   do
      if [ "$HELP_EXCLUDED" != -excluded ] ; then
         $HELP_FILE
      fi
   done

   SYNOPSIS_FILE_LIST="|"
   HELP_FILE_LIST=`ls -d ${HELP_LS_QUALIFIER}*`

   while read SYNOPSIS_LINE
   do
      HELP_FILE="${SYNOPSIS_LINE%%:*}"
      HELP_SYNOPSIS="${SYNOPSIS_LINE#*:*:}"
      HELP_SYNOPSIS="${HELP_SYNOPSIS# }"
      SYNOPSIS_FILE_LIST="${SYNOPSIS_FILE_LIST}$HELP_FILE|"

      # ignore fam.sh, rel.sh, repo.sh and src.sh since they are subsystems.
      if [ "${SUBSYSTEM_LIST#*$HELP_FILE}" != "$SUBSYSTEM_LIST" ] ; then
         continue
      fi

      EZHelpIsNotExcluded "$HELP_FILE"
      if [ $? -eq 0 ] ; then
         EZHelpFormatSynopsis "$HELP_FILE" "$HELP_SYNOPSIS"
      fi
   done <<!
`egrep $EGREP_OPTS "^#[ ]*Synopsis:|^'  Synopsis:" $HELP_FILE_LIST`
!

   # print out the names of the commands without synopsis's
   for HELP_FILE in $HELP_FILE_LIST
   do
      PIPE_HELP_FILE="|$HELP_FILE|"
      if [ "${SYNOPSIS_FILE_LIST#*$PIPE_HELP_FILE}" = \
           "$SYNOPSIS_FILE_LIST" ] ; then
         if [ ! -d "$HELP_FILE" ] ; then
            EZHelpIsNotExcluded "$HELP_FILE"
            if [ $? -eq 0 ] ; then
               EZHelpFormatSynopsis "$HELP_FILE" ""
           fi
         fi
      fi
   done

# TODO Get This Working...
if [ "$HELP_OUTPUT_FORMAT" != usage ] ; then
   if [ -f help.synopsis.include -a "$HELP_EXCLUDED" != -excluded ] ; then
      cat help.synopsis.include
   fi
fi
}

#===========================================================================
# Create the synopsis help file.
#===========================================================================
function EZHelpCreateSynopsisFile
{
   EZHelpCreateSynopsisList | sort > "$HELP_SYNOPSIS_FILE"
}

#===========================================================================
# Create the specified help in synopsis format.
#===========================================================================
function EZHelpCreateSynopsisHelp
{
   if [ -f help.synopsis.header ] ; then
      cat help.synopsis.header
   fi
   cat "$HELP_SYNOPSIS_FILE"
   if [ -f help.synopsis.footer ] ; then
      cat help.synopsis.footer
   fi
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -d) HELP_SCRIPTS_DIR="$2"
          shift
          shift ;;
      -excluded) HELP_EXCLUDED=$1
          shift ;;
      -s) HELP_SUBSYSTEM="$2"
          shift
          shift ;;
       *) if [ "$1" == "-?" ] ; then
             if [ "$HELP_COMMAND_NAME" = "" ] ; then
                HELP_OUTPUT_FORMAT=usage
             fi
          elif [ "$1" == "-??" ] ; then
             Usage
          else
             if [ "$HELP_COMMAND_NAME" = "" ] ; then
                HELP_COMMAND_NAME="$1"
             else
                HELP_COMMAND_NAME="$HELP_COMMAND_NAME $1"
             fi
             HELP_OUTPUT_FORMAT=command
          fi
          shift ;;
    esac
done

if [ ! -d "$HELP_SCRIPTS_DIR" ] ; then
   echo "ERROR: Failed to find help scripts dir $HELP_SCRIPTS_DIR."
   exit 1
fi
EZAddToPath $HELP_SCRIPTS_DIR "PREPEND"

cd $HELP_SCRIPTS_DIR

HELP_LS_QUALIFIER=
if [ "$HELP_SUBSYSTEM" != "" ] ; then
   HELP_LS_QUALIFIER="${HELP_SUBSYSTEM}_"
fi

EZHelpSetupExclusionList synopsis

EZHelpCreateSynopsisFile

HELP_FORMAT=synopsis
if [ "$HELP_OUTPUT_FORMAT" = synopsis ] ; then
   EZHelpCreateSynopsisHelp
fi

if [ "$HELP_OUTPUT_FORMAT" = command ] ; then
   egrep "^$HELP_COMMAND_NAME " "$HELP_SYNOPSIS_FILE"
   if [ $? -ne 0 ] ; then
      echo "ERROR: Failed to find help on command $HELP_COMMAND_NAME."
      exit 1
   fi
   echo
fi

#cat $HELP_SYNOPSIS_FILE
#exit 1
if [ "$HELP_OUTPUT_FORMAT" = usage -o "$HELP_OUTPUT_FORMAT" = command ] ; then
   HELP_FORMAT=usage
   EZHelpSetupExclusionList usage

   cat "$HELP_SYNOPSIS_FILE" | sort | cut -c1-16 > "$HELP_SYNOPSIS_FILE.fix"
   while read HELP_COMMAND
   do
      HELP_COMMAND_FINAL="$HELP_COMMAND"
      HELP_COMMAND_ONLY="${HELP_COMMAND#$HELP_SUBSYSTEM}"
      if [ "$HELP_COMMAND" != "$HELP_COMMAND_ONLY" ] ; then
         if [ "$HELP_SUBSYSTEM" = fam  -o \
              "$HELP_SUBSYSTEM" = rel  -o \
              "$HELP_SUBSYSTEM" = repo -o \
              "$HELP_SUBSYSTEM" = src ] ; then
            if [ -f "../${HELP_SUBSYSTEM}.ksh" ] ; then
               HELP_COMMAND_FINAL="${HELP_SUBSYSTEM}.ksh $HELP_COMMAND_ONLY"
            else
               HELP_COMMAND_FINAL="${HELP_SUBSYSTEM}.sh $HELP_COMMAND_ONLY"
            fi
         fi
      fi
      if [ "$HELP_OUTPUT_FORMAT" = command -a \
           "$HELP_COMMAND" != "$HELP_COMMAND_NAME" ] ; then
         continue
      fi
      if [ "$HELP_COMMAND" = dos2unix ] ; then
         HELP_COMMAND=dos2unix.sh
      fi
      if [ "$HELP_COMMAND" = db_create_object ] ; then
         HELP_COMMAND=db_create_objects
      fi
      
      if [ "${HELP_COMMAND% -?}" != "$HELP_COMMAND" ] ; then
         continue      # fam -?, rel -? and src -?
      fi
      if [ "$HELP_COMMAND" == "$HELP_SUBSYSTEM" ] ; then
         continue      # fam, rel and src
      fi
      if [ "$HELP_COMMAND" = help ] ; then
         help -??
         continue
      fi
      HELP_COMMAND_FILE=
      if [ -f "$HELP_COMMAND.ksh" ] ; then
         HELP_COMMAND_FILE="$HELP_COMMAND.ksh"
      fi
      if [ -f "$HELP_COMMAND.sh" ] ; then
         HELP_COMMAND_FILE="$HELP_COMMAND.sh"
      fi
      if [ -f "$HELP_COMMAND_FILE" ] ; then
         egrep $EZ_EGREP_QUIET "HelpType: SubSystemRoot" $HELP_COMMAND_FILE
         if [ $? -eq 0 ] ; then
            continue
         fi
      fi

      EZHelpIsNotExcluded "$HELP_COMMAND"
      if [ $? -eq 0 ] ; then
         eval $HELP_COMMAND_FINAL -?
      fi
   done < "$HELP_SYNOPSIS_FILE.fix"
   rm -f "$HELP_SYNOPSIS_FILE.fix"
fi

rm -f "$HELP_SYNOPSIS_FILE"

exit 0
