#===========================================================================
#  Filename: rel_refresh.ksh
#    Author: Craig Schneider
# Orig Date: 06/01/96
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Syncronize the deliver script
#===========================================================================

REL_LIST=
while [ "$#" -ne 0 ]
do
  case $1 in
    -a) ALL_RELEASES=TRUE
        shift ;;
    -f) FORCED=TRUE
        shift ;;
    -?) USAGE="TRUE"
        break;
        shift ;;
    *)  REL_LIST="$REL_LIST H:/release/$1"
        ALL_RELEASES=TRUE  # at least all those specified
        shift ;;
  esac
done

if [ "$USAGE" = "TRUE" ] ; then
   echo
   echo "Usage: $0 [RELEASE_NAME...] [-a] [-f]"
   echo
   echo "       -a: Check All Releases"
   echo "       -f: Forced Mode, Do Not Query"
   echo
   exit 1
fi

#if [ "$ALL_RELEASES" != "TRUE" ] ; then
#   echo
#   echo "Use: $0 -a     If You Want To Be Queried About ALL Possible Releases."
#   echo
#fi

function update_release
{
   LOCAL_DIR=$1
   REMOTE_DIR=$2

   if [ ! -d /release ] ; then
      mkdir /release
   fi
   rm -rf $LOCAL_DIR.old
   if [ -d $LOCAL_DIR ] ; then
      mv -f $LOCAL_DIR $LOCAL_DIR.old
      if [ $? -ne 0 ] ; then
         echo "ERROR: Failed refreshing $LOCAL_DIR"
         return
      fi
   fi
   cp -R $REMOTE_DIR $LOCAL_DIR
   echo "Successfully refreshed $LOCAL_DIR"
   if [ -f $LOCAL_DIR/rel_install.ksh ] ; then
      cd $LOCAL_DIR
      $LOCAL_DIR/rel_install.ksh
      cd -
   fi
}

if [ ! -d /release ] ; then
   echo
   echo "WARNING: $0 Is Expected To Be Run On A Drive Containing /release"
   echo
fi

if [ "$REL_LIST" = "" ] ; then
   if [ "$ALL_RELEASES" = "TRUE" ] ; then
      REL_LIST=`echo H:/release/*`
      if [ "$REL_LIST" = "H:/release/*" ] ; then
         echo "ERROR: Drive H: Is Not Connected."
         exit 1
      fi
   else
      REL_LIST=`echo /release/*`
      if [ "$REL_LIST" = "/release/*" ] ; then
         echo "ERROR: No Releases On This Drive Use -a For All Releases."
         exit 1
      fi
   fi
fi

for ONE_NAME in $REL_LIST
do
   REL_NAME=${ONE_NAME##*/}
   # ignore release names ending in .old
   if [ "$REL_NAME" != "${REL_NAME%.old}" ] ; then    
      continue
   fi
   REMOTE_DIR="H:/release/$REL_NAME"
   LOCAL_DIR=/release/$REL_NAME
   if [ ! -d $REMOTE_DIR ] ; then
      echo "Warning: Directory $REMOTE_DIR Not Found."
      continue
   fi
   if [ ! -d $LOCAL_DIR ] ; then
      if [ "$FORCED" = "TRUE" ] ; then
         update_release $LOCAL_DIR $REMOTE_DIR
      else
         if [ "$ALL_RELEASES" = "TRUE" ] ; then
            echo
            echo -n "Release $REL_NAME Does Not Exist Locally, Copy It <Y,N>? "
            read ANSWER
            if [ "$ANSWER" != "n" -a "$ANSWER" != "n" ] ; then
               update_release $LOCAL_DIR $REMOTE_DIR
            fi
         fi
      fi
      continue
   fi
   diff $REMOTE_DIR/deliver.log $LOCAL_DIR/deliver.log > $EZ_DEV_NULL 2>&1
   if [ $? -ne 0 ] ; then
      if [ "$FORCED" = "TRUE" ] ; then
         update_release $LOCAL_DIR $REMOTE_DIR
      else
         NEW_RELEASE=`grep "^Release " $REMOTE_DIR/deliver.log`
         echo -n "Please specify if $REL_NAME should be updated "
         echo -n "to $NEW_RELEASE <y,n> "
         read ANSWER
         if [ "$ANSWER" != "n" -a "$ANSWER" != "n" ] ; then
            update_release $LOCAL_DIR $REMOTE_DIR
         fi
      fi
   else
      echo "Successfully determined that release $REL_NAME is up to date."
   fi
done
exit 0
