#===========================================================================
#  Filename: rel_create.ksh
#    Author: Craig Schneider
# Orig Date: 07/12/01
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Create the specified release.
#===========================================================================

. $EZ_RELMGR_ENV

function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $1
   fi
   cat <<!

Usage: rel create [-NewWindow] [-Pause] [-ShowErrors] [-Timing] ReleaseName

   -NewWindow: Display the results in a new window.
       -pause: Pause for user input after creating the release.
  -ShowErrors: Show release creation errors without prompting.
      -Timing: Display timing information on some messages.
  ReleaseName: Release name to be created, possibilities include:
               $(EZRelNameLongList ", ")

!
   exit 1
}

PAUSE=FALSE
REMOTE_RUN_ARGS=
TIMING=FALSE

typeset -l LOWERCASE_ARG
while [ "$#" -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
     -newwindow) REMOTE_RUN_ARGS="$REMOTE_RUN_ARGS -NewWindow"
                 shift ;;
         -pause) PAUSE=TRUE
                 shift ;;
    -showerrors) BUILD_OPTS="$BUILD_OPTS -ShowErrors"
                 shift ;;
        -timing) TIMING=TRUE
                 shift ;;
             -?) Usage ;;
              *) RELEASE_NAME=$1
                 shift ;;
   esac
done

ALLOW_MULTI_REL=TRUE
RELEASE_NAME=`EZRelNameSelect "$RELEASE_NAME"`
#echo "RELEASE_NAME=$RELEASE_NAME"
REL_FILE_NAME=`EZRelGetFileByName "$RELEASE_NAME"`
#echo "REL_FILE_NAME=$REL_FILE_NAME"

if [ "$REL_FILE_NAME" = "" ] ; then
   echo "ERROR: Failed to determine .rel file name for release $RELEASE_NAME."
   exit 1
fi
src.ksh extract -quiet "$REL_FILE_NAME"

MULTI_RELEASE_LIST=`EZRelMultiReleaseList "$REL_FILE_NAME"`
if [ "$MULTI_RELEASE_LIST" != "" ] ; then
   for SINGLE_REL_NAME in $MULTI_RELEASE_LIST
   do
      rel.sh create -NewWindow -Timing $SINGLE_REL_NAME
   done
   echo "[$(date '+%D %T')] \c"
   echo "Successfully finished creating release $RELEASE_NAME."
   exit $?
fi

FILE_IS_EXECUTABLE=`EZRelFileIsExecutable "$REL_FILE_NAME"`
if [ "$FILE_IS_EXECUTABLE" = TRUE ] ; then
   REL_FILE_NAME_KSH=`EZPath2Ksh $REL_FILE_NAME`
   $REL_FILE_NAME_KSH
   exit $?
fi

REMOTE_REL_CREATE_HOST=""
if [ "$REL_CREATE_HOSTS" != "" ] ; then
   IS_REL_CREATE_HOST=FALSE
   for SINGLE_HOST in $REL_CREATE_HOSTS
   do
      if [ "$REMOTE_REL_CREATE_HOST" = "" ] ; then
         REMOTE_REL_CREATE_HOST="$SINGLE_HOST"
      fi
      if [ "$EZ_COMPUTERNAME" = "$SINGLE_HOST" ] ; then
         IS_REL_CREATE_HOST=TRUE
      fi
   done
else
   IS_REL_CREATE_HOST=TRUE
fi

if [ "$IS_REL_CREATE_HOST" = TRUE ] ; then
#   if [ "$EZ_KSH_BASE" = CYGWIN ] ; then
#      eval C4.sh ProcessStatus -s None -BadShell -Quiet -WhilePPID $$ &
#      echo "Successfully started the BadShell monitor executable."
#   fi

   cd "$EZ_FAMILYTREE_ROOT"
   src.ksh test mk.sh
   if [ $? -ne 0 ] ; then
      echo "ERROR: Unable to communicate with the Source Control server."
      exit 1
   fi
   src.ksh refresh

   rel.sh build $BUILD_OPTS -f $REL_FILE_NAME
   EXIT_CODE=$?
else
   if [ "$TIMING" = TRUE ] ; then
      echo "[$(date '+%D %T')] \c"
   fi
   echo "Successfully creating release $RELEASE_NAME \c"
   echo "on $REMOTE_REL_CREATE_HOST."
   RemoteRun.ksh -h $REMOTE_REL_CREATE_HOST $REMOTE_RUN_ARGS \
                  rel.sh create -ShowErrors $RELEASE_NAME
   EXIT_CODE=$?
fi

if [ "$PAUSE" = TRUE ] ; then
   echo
   echo
   echo -n "*********** Press enter to exit rel create $RELEASE_NAME "
   read answer
fi

exit $EXIT_CODE
