#===========================================================================
#  Filename: rel_extract.ksh
#    Author: Craig Schneider
# Orig Date: 02/14/01
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Extract the specified release.
#===========================================================================

. $EZ_SHARED_ENV

if [ "$1" = -? ] ; then
   echo
   echo "Usage: extract [RELEASE_NAME]"
   echo
   exit 1
fi

if [ "${PWD#*/}" = "Source" ] ; then
   echo
   echo    "You Are In The Source Directory,"
   echo -n "Are You Sure That You Know What You Are Doing? <Y,N> "
   read ANSWER
   if [ "$ANSWER" = "N" -o "$ANSWER" = "n" ] ; then
      return
   fi
fi

rel_pick $*
  
egrep "SOURCED_LIST=" $EZ_FAMILYTREE_ROOT/Releases/Built/$THE_RELEASE.ksh \
                         > $TMPDIR/rel.tmp
. $TMPDIR/rel.tmp

if [ "$SOURCE_CONTROL_SYSTEM" = vss ] ; then
   for A_NAME in $SOURCED_LIST
   do
      mkdir $A_NAME
      \cd $A_NAME
      # NOTE: the space after -V is imparative for 3rdParty, because when
      # the label starts with a number it must be preceeded by a space.

      VSS_OUTPUT_FILE="$(EZPath2Win $TMPDIR/ss_refresh.$$)"
      ss Get $EZ_RELEASE_ROOT/$A_NAME -R -i-Y -V $THE_RELEASE $EZ_VSS_OPTIONS \
             "-O&$VSS_OUTPUT_FILE"
      cat "$VSS_OUTPUT_FILE" | \
            sed -e '/as the default folder/d' \
                -e '/^Getting /d' \
                -e '/^Replacing local copy of /d' \
                -e '/ is not an existing filename or project/d' \
                -e '/^\$\//d'
      rm -f "$VSS_OUTPUT_FILE"
      \cd ..
      echo "Successfully extracted family $A_NAME"
   done
elif [ "$SOURCE_CONTROL_SYSTEM" = svn ] ; then
   for A_NAME in $SOURCED_LIST
   do
      mkdir -p $A_NAME
      svn checkout "$EZ_SOURCE_CONTROL_ROOT/Releases/$THE_RELEASE/$A_NAME" | \
          sed -e 's/Checked out revision/Successfully checked out revision/' \
              -e 's/^A    /Successfully extracted /'
   done
else
   echo "ERROR: Unsupported Source Control System $SOURCE_CONTROL_SYSTEM for extracting."
   exit 1
fi
exit 0
