#===========================================================================
#  Filename: rel_select.ksh
#    Author: Craig Schneider
# Orig Date: 02/14/01
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Select a release name from the list.
#===========================================================================

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: select [-a] [-f SelectionFile] [-list] [-v] [-prefix ReleasePrefix]


        -a: Select the release from all possible releases.
     -list: List the built releases.
        -v: Verbose output is displayed.


Usage: set [-a | ReleaseName]

        -a: Set the release from all possible releases.

!
   # Must also spit out Usage for 'rel set' which is handled by rel internally
   exit 1
}

if [ "$1" = -? ] ; then
   Usage
fi

FOR_DESTROY=FALSE
ALL_RELEASES=FALSE
SELECTION_FILE=
LIST_MODE=FALSE
RELEASE_NAME=
RELEASE_PREFIX=
REL_NAME_LIST=
VERBOSE_MODE=FALSE
TOTAL_REFERENCES=0
TOTAL_RELEASES=0
while [ $# -ne 0 ]
do
   case $1 in
      -a) ALL_RELEASES=TRUE
          shift ;;
      -f) SELECTION_FILE="$2"
          shift
          shift ;;
-for_destroy) FOR_DESTROY=TRUE
          shift ;;
   -list) LIST_MODE=TRUE
          shift ;;
 -prefix) RELEASE_PREFIX="$2"
          shift
          shift ;;
      -v) VERBOSE_MODE=TRUE
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) RELEASE_NAME=$1
          REL_NAME_LIST="$REL_NAME_LIST $1"
          shift ;;
    esac
done

if [ "$RELEASE_NAME" = "" -o "$LIST_MODE" = TRUE ] ; then
   SAVE_PWD="$PWD"
   if [ "$REL_NAME_LIST" = "" ] ; then
      if [ "$LIST_MODE" != TRUE ] ; then
         if [ -d $EZ_FAMILYTREE_ROOT/Releases/Open ] ; then
            \cd  "$(Path2Ksh.ksh $EZ_FAMILYTREE_ROOT/Releases/Open)"
            REL_NAME_LIST="$REL_NAME_LIST "`echo $RELEASE_PREFIX*.ksh | \
                                            sed 's/\.ksh//g'`
         fi
      fi
      if [ "$LIST_MODE" = TRUE -o "$ALL_RELEASES" = TRUE ] ; then
         if [ -d $EZ_FAMILYTREE_ROOT/Releases/Built ] ; then
            \cd  "$(Path2Ksh.ksh $EZ_FAMILYTREE_ROOT/Releases/Built)"
            REL_NAME_LIST="$REL_NAME_LIST "`echo $RELEASE_PREFIX*.ksh | \
                                            sed 's/\.ksh//g'`
         fi
      fi
   fi

   if [ -d $EZ_FAMILYTREE_ROOT/Releases/Built ] ; then
      \cd  "$(Path2Ksh.ksh $EZ_FAMILYTREE_ROOT/Releases/Built)"
   fi

   if [ "$LIST_MODE" != "TRUE" ]; then
      PS3="Select Release Name: "
      select RELEASE_NAME in $REL_NAME_LIST
      do
         if [ "$RELEASE_NAME" != "" ] ; then
            RELEASE_NAME=${RELEASE_NAME%%...*}
            break
         fi
      done
   else
      if [ "$FOR_DESTROY" = FALSE ] ; then
         cat <<!
Release Name                  References
============================  ================================================
!
      fi
      typeset -L29 FMT_REL_NAME
      typeset -R2  FMT_REF_COUNT
      FMT_PRE_REF=""
      REL_REF_OUT_FILE="$TMPDIR/rel_select.$$"
      REL_XREF_FILE="$TMPDIR/rel_select_xref.$$"
      fgrep "RELEASED_LIST=" *.ksh | grep -v "LIST=' '" | \
                                     sed 's/.ksh//'  > "$REL_XREF_FILE"
      fgrep "BUILT_LIST="    *.ksh | grep -v "LIST=' '" | \
                                     sed 's/.ksh//' >> "$REL_XREF_FILE"
      for REL_NAME in $REL_NAME_LIST
      do
         rm -f "$REL_REF_OUT_FILE"
	 touch "$REL_REF_OUT_FILE"

         ARCHIVE_STATUS=
         LOCK_STATUS=

         REL_KSH_FILE="$REL_NAME.ksh"
         if [ -f $REL_KSH_FILE ] ; then
            egrep $EZ_EGREP_QUIET "^LOCKED=TRUE" $REL_KSH_FILE
            if [ $? -eq 0 ] ; then
               LOCK_STATUS="Locked"
            fi
            egrep $EZ_EGREP_QUIET "^ARCHIVED=TRUE" $REL_KSH_FILE
            if [ $? -eq 0 ] ; then
               ARCHIVE_STATUS="Archived"
            fi
            grep "^LOGICAL_REFERENCE=" $REL_KSH_FILE | cut -f2 -d "'" \
                 >> "$REL_REF_OUT_FILE"
         fi

         FMT_REF_COUNT=`wc -l "$REL_REF_OUT_FILE" | awk '{print $1}'`
         REF_STATUS="$FMT_REF_COUNT References    "
         let TOTAL_REFERENCES=TOTAL_REFERENCES+FMT_REF_COUNT
         if [ $FMT_REF_COUNT -eq 1 ] ; then
            REF_STATUS="$FMT_REF_COUNT Reference     "
         elif [ $FMT_REF_COUNT -eq 0 ] ; then
            for PREV_NUM in 0 1 2 3
            do
               # 11/18/2013 finding releases which shared the same prefix
               #LAST_REL_NAME=`last_name.ksh -r ${REL_NAME%%_*} -$PREV_NUM`
               LAST_REL_NAME=`last_name.ksh -r ${REL_NAME%_*} -$PREV_NUM`
               if [ "$LAST_REL_NAME" = "$REL_NAME" ] ; then
                  break
               fi
            done
            if [ $PREV_NUM -eq 0 ] ; then
               REF_STATUS="Last Build "
               let TOTAL_REFERENCES=TOTAL_REFERENCES+FMT_REF_COUNT
            elif [ $PREV_NUM -lt 3 ] ; then
               REF_STATUS="Last Build -$PREV_NUM "
               let TOTAL_REFERENCES=TOTAL_REFERENCES+FMT_REF_COUNT
            else
               REF_STATUS="UNREFERENCED  "
            fi
         fi

         FMT_REL_NAME="$REL_NAME"
         echo "$FMT_REL_NAME $REF_STATUS $LOCK_STATUS $ARCHIVE_STATUS"
         if [ "$VERBOSE_MODE" = TRUE ] ; then # the typeset -L22 affects this
            awk '{ print "                              " $0 }' \
                   "$REL_REF_OUT_FILE"
         fi
         let TOTAL_RELEASES=TOTAL_RELEASES+1
      done
      if [ "$VERBOSE_MODE" = TRUE -a "$FOR_DESTROY" = FALSE ] ; then
         echo -n "Successfully tracking $TOTAL_REFERENCES references "
         echo    "against $TOTAL_RELEASES Releases."
      fi
      echo
      rm -f "$REL_REF_OUT_FILE"
      rm -f "$REL_XREF_FILE"
   fi

   \cd "$(Path2Ksh.ksh $SAVE_PWD)"
fi

if [ "$SELECTION_FILE" != "" ] ; then
   echo "export THE_RELEASE=$RELEASE_NAME" >> $SELECTION_FILE
fi

if [ "$FOR_DESTROY" = TRUE -a "$LOCK_STATUS" = Locked ] ; then
   exit 2
elif [ $TOTAL_REFERENCES -gt 0 ] ; then
   exit 1
else
   exit 0
fi
