#===========================================================================
#  Filename: relmgr_env.ksh
#    Author: Craig Schneider
# Orig Date: 08/30/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Provides the environment config info for the Rel suite.
#===========================================================================

. $EZ_SHARED_ENV

if [ ! -f "$EZ_FAMILYTREE_ROOT/Config/relmgr_config.ksh" ] ; then
   echo "ERROR: Failed to find config file $EZ_FAMILYTREE_ROOT/Config/relmgr_config.ksh."
   exit 1
fi
. "$EZ_FAMILYTREE_ROOT/Config/relmgr_config.ksh"

function EZRelFileIsExecutable
{
   REL_FILE_NAME="$1"

   eval `egrep "^REL_FILE_IS_EXECUTABLE=" "$REL_FILE_NAME"`
   echo "$REL_FILE_IS_EXECUTABLE"
}

function EZRelFileList
{
   if [ "$ALLOW_MULTI_REL" = TRUE ] ; then
      find "${EZ_FAMILYTREE_ROOT%/*}" -maxdepth 3 \
            \( -name \*.rel -o -name \*.mrel \) -print
   else
      find "${EZ_FAMILYTREE_ROOT%/*}" -maxdepth 3 -name \*.rel -print
   fi
}

function EZRelMultiReleaseList
{
   REL_FILE_NAME="$1"

   eval `egrep "^REL_MULTI_RELEASE_LIST=" "$REL_FILE_NAME"`
   echo "$REL_MULTI_RELEASE_LIST"
}

function EZRelNameList
{
   DELIM="$1"
   if [ "$DELIM" = "" ] ; then
      DELIM=" "
   fi
   REL_NAME_LIST=
   for REL_FILE_NAME in `EZRelFileList`
   do
      REL_NAME=
      eval `egrep "^REL_NAME=" "$REL_FILE_NAME"`
      if [ "$REL_NAME" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a REL_NAME setting." 1>&2
      else
         REL_NAME_LIST="$REL_NAME_LIST$DELIM$REL_NAME"
      fi
   done
   echo "${REL_NAME_LIST#$DELIM}"
}

function EZRelNameLongList
{
   DELIM="$1"
   if [ "$DELIM" = "" ] ; then
      DELIM=" "
   fi
   REL_NAME_LONG_LIST=
   for REL_FILE_NAME in `EZRelFileList`
   do
      REL_NAME_LONG=
      eval `egrep "^REL_NAME_LONG=" "$REL_FILE_NAME"`
      if [ "$REL_NAME_LONG" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a REL_NAME_LONG setting." 1>&2
      else
         REL_NAME_LONG_LIST="$REL_NAME_LONG_LIST$DELIM$REL_NAME_LONG"
      fi
   done
   echo "${REL_NAME_LONG_LIST#$DELIM}"
}

function EZRelNukeNameList
{
   REL_NUKE_FILE="$TMPDIR/rel_nuke.$$"
   REL_NUKE_SORTED_FILE="$TMPDIR/rel_nuke.sorted.$$"
   rm -f "$REL_NUKE_FILE" "$REL_NUKE_SORTED_FILE"
   for REL_FILE_NAME in `EZRelFileList`
   do
      REL_NAME_LONG=
      REL_NAME_NUKE=
      BUILD_NUMBER_SYNC_NAME=
      BUILD_NUMBER_SYNC_ORDER=
      eval `egrep "^REL_NAME_LONG=|^REL_NAME_NUKE=|^BUILD_NUMBER_SYNC_NAME=|^BUILD_NUMBER_SYNC_ORDER=" "$REL_FILE_NAME"`
      if [ "$REL_NAME_LONG" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a REL_NAME_LONG setting." 1>&2
      elif [ "$BUILD_NUMBER_SYNC_NAME" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a BUILD_NUMBER_SYNC_NAME setting." 1>&2
      elif [ "$BUILD_NUMBER_SYNC_ORDER" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a BUILD_NUMBER_SYNC_ORDER setting." 1>&2
      else
         if [ "$REL_NAME_NUKE" = "" ] ; then
            REL_NAME_NUKE="$REL_NAME_LONG"
         fi 
         echo "$BUILD_NUMBER_SYNC_ORDER $BUILD_NUMBER_SYNC_NAME $REL_NAME_NUKE" >> "$REL_NUKE_FILE"
      fi
   done
   sort -rn "$REL_NUKE_FILE" > "$REL_NUKE_SORTED_FILE"
   REL_NUKE_NAME_LIST=`cat "$REL_NUKE_SORTED_FILE" | awk '{ print $3 }' | \
                       tr '\n' ' '`
   rm -f "$REL_NUKE_FILE" "$REL_NUKE_SORTED_FILE"
   echo "$REL_NUKE_NAME_LIST"
}

function EZRelNameSelect
{
   RELEASE_NAME="$1"
   if [ "$RELEASE_NAME" = "" ] ; then
      PS3="Select Release Name: "
      select RELEASE_NAME in `EZRelNameLongList`
      do
         if [ "$RELEASE_NAME" != "" ] ; then
            RELEASE_NAME=${RELEASE_NAME%%...*}
            break
         fi
      done
   else
      REL_FILE_NAME=`EZRelGetFileByName "$RELEASE_NAME"`
      if [ -f "$REL_FILE_NAME" ] ; then
         eval `egrep "^REL_NAME_LONG=" "$REL_FILE_NAME"`
         if [ "$REL_NAME_LONG" != "" ] ; then
            RELEASE_NAME="$REL_NAME_LONG"
         fi
      fi
   fi
   echo "$RELEASE_NAME"
}

function EZRelGetFileByName
{
   RELEASE_NAME="$1"

   for REL_FILE_NAME in `EZRelFileList`
   do
      REL_NAME=
      REL_NAME_LONG=
      eval `egrep "^REL_NAME=|^REL_NAME_LONG=" "$REL_FILE_NAME"`
      if [ "$REL_NAME" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a REL_NAME setting." 1>&2
      elif [ "$REL_NAME_LONG" = "" ] ; then
         echo -n "ERROR: File $REL_FILE_NAME does not contain " 1>&2
         echo    "a REL_NAME_LONG setting." 1>&2
      else
         typeset -u RELEASE_NAME_UPPER="$RELEASE_NAME"
         typeset -u REL_NAME_UPPER="$REL_NAME"
         typeset -u REL_NAME_LONG_UPPER="$REL_NAME_LONG"
         if [ "$RELEASE_NAME_UPPER" = "$REL_NAME_UPPER" -o \
              "$RELEASE_NAME_UPPER" = "$REL_NAME_LONG_UPPER" ] ; then
            echo "$REL_FILE_NAME"
         fi
      fi
   done
}
