#===========================================================================
#  FileName: set_debugs.ksh
#    Author: Craig Schneider
# Orig Date: 08/25/03
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Interactive tool to help with setting debug env variables.
#===========================================================================

if [ "$1" = -? ] ; then
   cat <<!

Usage: set_debugs [-v] [DebugOption(s)...]

            -v: Display verbose messages about operations being performed.
DebugOption(s): String of debug commands to perform as if the user keyed them.

!
   if [ "$-" = "${-%i*}" ] ; then
      exit 1
   fi
   return
fi

if [ "$-" = "${-%i*}" ] ; then
   echo
   echo "ERROR: \$- set to $-"
   echo "ERROR: $0 should be dotted and not executed.  Try using 'debugs'."
   echo
   exit 1
fi


if [ "${1}" != "${1#*/}" ] ; then
   shift    # shift off path junk on first call, ???WHY???
fi

VERBOSE_MODE=FALSE
if [ "$1" = "-v" ] ; then
   VERBOSE_MODE=TRUE
fi

FS=","

function DisableCategory
{
   CATEGORY="$1"
   export DebugCategories=${DebugCategories%$FS$CATEGORY*}${DebugCategories#*$FS$CATEGORY}
   if [ "$DebugCategories" = "$FS" ] ; then
      unset DebugCategories
   fi
}

function EnableCategory
{
   CATEGORY="$1"
   if [ "$DebugCategories" = "" -o "$DebugCategories" = "NONE" ] ; then
      export DebugCategories="$FS$CATEGORY$FS"
   else
      export DebugCategories="$DebugCategories$CATEGORY$FS"
   fi
}

# Calling the CategoryEnabled function is way too slow
#function CategoryEnabled
#{
#   if [ "$DebugCategories" != "${DebugCategories%$FS$1$FS*}" ] ; then
#      echo "ON"
#   else
#      echo ""
#   fi
#}


CATEGORY_LIST="CACHE CONST DEBUG EVENT FTP GET MENU REPLY SET SQL SLOW TRACE WARN"
VCATEGORY_LIST="VCACHE VCONST VDEBUG VEVENT VFTP VGET VVGET VSET VSQL VTRACE"
while true
do
   #######################################################################
   # LIST CURRENT DEBUG CATEGORY SETTINGS AND COMMANDS
   #######################################################################

   typeset -u COMMAND_LIST
   COMMAND_LIST=
   if [ "$1" = "" ] ; then
      if [ "$DebugToConsole" != "" ] ; then
         eval CONSOLE_TEXT=\"C\) Console output is ENABLED.\"
      else
         eval CONSOLE_TEXT=\"C\) Console output is disabled.\"
      fi

      if [ "$DebugFileName" != "" ] ; then
         eval FILE_TEXT=\"F\) File output is set to $DebugFileName.\"
      else
         eval FILE_TEXT=\"F\) File output is disabled.\"
      fi

      if [ "$DebugToWindow" != "" ] ; then
         eval WINDOW_TEXT=\"W\) Window output is ENABLED.\"
      else
         eval WINDOW_TEXT=\"W\) Window output is disabled.\"
      fi

      if [ "$DebugFormatString" != "" ] ; then
         eval FORMAT_STRING_TEXT=\"T\) Text format string is $DebugFormatString.\"
      else
         eval FORMAT_STRING_TEXT=\"T\) Text format string is unset.\"
      fi

       ENABLED_CATEGORY_TEXT="ON"
      DISABLED_CATEGORY_TEXT="--"  # @@@off
      typeset -L3 FORMAT_NUMBER
      typeset -L7 FORMAT_NAME
      typeset -L2 FORMAT_ENABLED
      ITEM_NUMBER=0
      for SINGLE_CATEGORY in $CATEGORY_LIST
      do
         let ITEM_NUMBER=ITEM_NUMBER+1
         CATEGORY_ENABLED=
         if [ "$DebugCategories" != "${DebugCategories%$FS$SINGLE_CATEGORY$FS*}" ] ; then
            CATEGORY_ENABLED="$ENABLED_CATEGORY_TEXT"
         elif [ "$DebugCategories" != "" ] ; then
            CATEGORY_ENABLED="$DISABLED_CATEGORY_TEXT"
         fi

         FORMAT_NUMBER="${ITEM_NUMBER})@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_NAME="${SINGLE_CATEGORY}@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_ENABLED="${CATEGORY_ENABLED}@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_ITEM="$ITEM_NUMBER"
         if [ $ITEM_NUMBER -lt 10 ] ; then
            FORMAT_ITEM="0$ITEM_NUMBER"
         fi
         eval FORMAT_CAT_$FORMAT_ITEM=\"$FORMAT_NUMBER $FORMAT_NAME $FORMAT_ENABLED\"
      done

      VCATEGORY_NUMBER=0
      for SINGLE_VCATEGORY in $VCATEGORY_LIST
      do
         let VCATEGORY_NUMBER=VCATEGORY_NUMBER+1
         let ITEM_NUMBER=ITEM_NUMBER+1
         typeset -L3 FORMAT_NUMBER
         typeset -L7 FORMAT_NAME
         typeset -L2 FORMAT_ENABLED

         PRETTY_COMMAND="$ITEM_NUMBER"
         PRETTY_VCATEGORY="$SINGLE_VCATEGORY"
         VCATEGORY_ENABLED=
         if [ "$DebugCategories" != "${DebugCategories%$FS$SINGLE_VCATEGORY$FS*}" ] ; then
            VCATEGORY_ENABLED="$ENABLED_CATEGORY_TEXT"
         elif [ "$DebugCategories" != "" ] ; then
            VCATEGORY_ENABLED="$DISABLED_CATEGORY_TEXT"
         fi
         FORMAT_NUMBER="${ITEM_NUMBER})@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_NAME="${SINGLE_VCATEGORY}@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_ENABLED="${VCATEGORY_ENABLED}@@@@@@@@@@@@@@@@@@@@@@@@@@"
         FORMAT_VCAT_NUM="$VCATEGORY_NUMBER"
         if [ $VCATEGORY_NUMBER -lt 10 ] ; then
            FORMAT_VCAT_NUM="0$VCATEGORY_NUMBER"
         fi
         eval FORMAT_VCAT_$FORMAT_VCAT_NUM=\"$FORMAT_NUMBER $FORMAT_NAME $FORMAT_ENABLED\"
      done

      PAD_CHAR="@"
( cat | sed "s/$PAD_CHAR/ /g" ) <<!

  Commands                             Debug Output Streams 
  ================================     =================================
  E) Enable all categories.            $CONSOLE_TEXT
  D) Disable all categories.           $FILE_TEXT
  S) Enable standard categories.       $WINDOW_TEXT
  V) View all debug settings.          $FORMAT_STRING_TEXT
  U) Unset all debug settings.
  Q) Quit.                            
                                      
  Standard Debug Categories            Verbose Debug Categories
  ================================     =================================
  $FORMAT_CAT_01    $FORMAT_CAT_08     $FORMAT_VCAT_01     $FORMAT_VCAT_06
  $FORMAT_CAT_02    $FORMAT_CAT_09     $FORMAT_VCAT_02     $FORMAT_VCAT_07
  $FORMAT_CAT_03    $FORMAT_CAT_10     $FORMAT_VCAT_03     $FORMAT_VCAT_08
  $FORMAT_CAT_04    $FORMAT_CAT_11     $FORMAT_VCAT_04     $FORMAT_VCAT_09
  $FORMAT_CAT_05    $FORMAT_CAT_12     $FORMAT_VCAT_05     $FORMAT_VCAT_10
  $FORMAT_CAT_06    $FORMAT_CAT_13     
  $FORMAT_CAT_07                      

!
      if [ "$DebugCategories" != "" -a "$DebugCategories" != "NONE" -a \
           "$DebugToConsole"   = "" -a "$DebugToWindow"    =  "" -a    \
           "$DebugFileName"    = "" ] ; then
         echo "Warning: Categories are enabled but no output stream is specified."
         echo
      fi
      echo -n "Please specify one or more commands: "
      read COMMAND_LIST
   else
      COMMAND_LIST="$1"
      shift
   fi

   #######################################################################
   # PROCESS COMMANDS
   #######################################################################

   for SINGLE_COMMAND in $COMMAND_LIST
   do

   if [ "$SINGLE_COMMAND" = D ] ; then
      export DebugCategories="NONE"
      unset DebugToConsole
   elif [ "$SINGLE_COMMAND" = C ] ; then
      if [ "$DebugToConsole" != "" ] ; then
         unset DebugToConsole
      else
         export DebugToConsole=TRUE
      fi
   elif [ "$SINGLE_COMMAND" = F ] ; then
      unset DebugFileName
      echo
      echo -n "Enter the debug file name: " 
      read LOG_FILE_NAME
      if [ "$LOG_FILE_NAME" != "" ] ; then
         export DebugFileName="$LOG_FILE_NAME"
      fi
   elif [ "$SINGLE_COMMAND" = W ] ; then
      if [ "$DebugToWindow" != "" ] ; then
         unset DebugToWindow
      else
         export DebugToWindow=TRUE
      fi
   elif [ "$SINGLE_COMMAND" = T ] ; then
      unset  DebugFormatString
      cat <<!

Default format string is: {0,5}ms {1,4}ms {2,-5} {3,-13} {4,-17} {5}

Where: {0} is Running Milliseconds
       {1} is Elapsed Milliseconds
       {2} is Message Category
       {3} is Message Object
       {4} is Message Identifier
       {5} is Message Text

!
      echo -n "Enter the debug format string: " 
      read FORMAT_STRING
      if [ "$FORMAT_STRING" != "" ] ; then
         export DebugFormatString="$FORMAT_STRING"
      fi
   elif [ "$SINGLE_COMMAND" = V -o "$VERBOSE_MODE" = TRUE ] ; then
      echo
      echo "DebugFileName=$DebugFileName"
      echo "DebugFormatString=$DebugFormatString"
      echo "DebugToConsole=$DebugToConsole"
      echo "DebugToWindow=$DebugToWindow"
      echo "DebugCategories=$DebugCategories"
   elif [ "$SINGLE_COMMAND" = U ] ; then
      unset DebugCategories
      unset DebugFileName
      unset DebugFormatString
      unset DebugToConsole
      unset DebugToWindow
   elif [ "$SINGLE_COMMAND" = Q -o "$SINGLE_COMMAND" = QUIT ] ; then
      return
   else
      ITEM_NUMBER=0
      for SINGLE_CATEGORY in $CATEGORY_LIST
      do
         let ITEM_NUMBER=ITEM_NUMBER+1
         if [ "$DebugCategories" != "${DebugCategories%$FS$SINGLE_CATEGORY$FS*}" ] ; then
            if [ "$SINGLE_COMMAND" = $ITEM_NUMBER -o \
                 "$SINGLE_COMMAND" = "$SINGLE_CATEGORY" ] ; then
               if [ "$SINGLE_COMMAND" != E ] ; then
                  DisableCategory "$SINGLE_CATEGORY"
               fi
            fi
         else
            if [ "$SINGLE_COMMAND" = E -o \
                 "$SINGLE_COMMAND" = S -o \
                 "$SINGLE_COMMAND" = $ITEM_NUMBER -o \
                 "$SINGLE_COMMAND" = "$SINGLE_CATEGORY" ] ; then
               EnableCategory "$SINGLE_CATEGORY"
            fi
         fi
      done
      for SINGLE_VCATEGORY in $VCATEGORY_LIST
      do
         let ITEM_NUMBER=ITEM_NUMBER+1
         if [ "$DebugCategories" != "${DebugCategories%$FS$SINGLE_VCATEGORY$FS*}" ] ; then
            if [ "$SINGLE_COMMAND" = $ITEM_NUMBER -o \
                 "$SINGLE_COMMAND" = "$SINGLE_VCATEGORY" ] ; then
               if [ "$SINGLE_COMMAND" != E ] ; then
                  DisableCategory "$SINGLE_VCATEGORY"
               fi
            fi
         else
            if [ "$SINGLE_COMMAND" = E -o \
                 "$SINGLE_COMMAND" = $ITEM_NUMBER -o \
                 "$SINGLE_COMMAND" = "$SINGLE_VCATEGORY" ] ; then
               EnableCategory "$SINGLE_VCATEGORY"
            fi
         fi
      done
   fi
   done
done
