#===========================================================================
#  Filename: src_patch_file.ksh
#    Author: Craig Schneider
# Orig Date: 09/06/13
# Copyright: ShadowScape Technologies, Inc. (see copyright file for details)
#  Synopsis: Patch a file from one source control hierarchy to another.
#===========================================================================

. $EZ_SOURCE_ENV

#===========================================================================
# Display the usage.
#===========================================================================
function Usage
{
   if [ "$1" != "" ] ; then
      echo
      echo $*
   fi
   cat <<!

Usage: src patch_file FileName(s)... [-d Directory]
                   [-dryrun] [-quiet | -verbose] [-yes]

          -d: The directory containing the source file.
     -dryrun: Display the commands to be executed, without executing them.
 FileName(s): One or more file names to patch from another hierarchy.
          -m: The message to be used as the reason for checkin.
      -quiet: Suppress non critical messages about operations being performed.
    -verbose: Display verbose messages about operations being performed.
        -yes: All user questions will be implicitly answered with a yes.

!
   exit 1
}

#===========================================================================
# Process the command line arguments.
#===========================================================================
if [ "$1" = -? ] ; then
   Usage
fi
if [ $# -eq 0 ] ; then
   Usage
fi

integer SRC_FILE_LIST_COUNT=0
EXIT_CODE=0

EZ_DRYRUN=FALSE
EZ_VERBOSITY=
MESSAGE_TEXT="src patch file"
SRC_OPTS=
STABALIZE_ARGS=
SOURCE_DIRECTORY=

typeset -l LOWERCASE_ARG
while [ $# -ne 0 ]
do
   LOWERCASE_ARG="$1"
   case $LOWERCASE_ARG in
      -d) SOURCE_DIRECTORY="$2"
          shift
          shift ;;
      -dryrun)
          EZ_DRYRUN=TRUE
          shift ;;
      -m) MESSAGE_TEXT="$2"
          shift
          shift ;;
      -quiet|-verbose)
          EZ_VERBOSITY="${LOWERCASE_ARG#-}"
          SRC_OPTS="$SRC_OPTS $1"
          shift ;;
      -yes)
          STABALIZE_ARGS="$STABALIZE_ARGS $1"
          shift ;;
      -*) if [ "$1" != "-?" ] ; then
             Usage "Invalid argument: $1"
          else
             Usage
          fi ;;
       *) SRC_FILE_NAME_LIST[SRC_FILE_LIST_COUNT]="$1"
          SRC_FILE_LIST_COUNT=SRC_FILE_LIST_COUNT+1
          shift ;;
    esac
done

#===========================================================================
# Perform the requested source control operations.
#===========================================================================
integer COUNT=0
while [ $COUNT -lt $SRC_FILE_LIST_COUNT ]
do
   SRC_FILE_NAME="${SRC_FILE_NAME_LIST[COUNT]}"
   COUNT=COUNT+1

   TOP_ROOT="${EZ_FAMILYTREE_ROOT%/*}"
   PWD_WIN=`EZPath2Win "$PWD"`
   DIR_FROM_TOP="${PWD_WIN#$TOP_ROOT}"
   DIR_FROM_TOP="${DIR_FROM_TOP#/}"
   SYNC_TOP="C:/Source"
   if [ ! -d "$SYNC_TOP" -a -d "C:/TeoDev/Source" ] ; then
      SYNC_TOP="C:/TeoDev/Source"
   fi
   if [ ! -d "$SYNC_TOP" ] ; then
      if [ "$EZ_COMPUTERNAME" != "${EZ_COMPUTERNAME#PCC-BUILD}" -a \
           -d P:/Development/Source ] ; then
         SYNC_TOP="P:/Development/Source"
      fi
   fi
   if [ "$SOURCE_DIRECTORY" != "" ] ; then
      SYNC_FILE_FULL_NAME="$SOURCE_DIRECTORY/$SRC_FILE_NAME"
   else
      SYNC_FILE_FULL_NAME="$SYNC_TOP/$DIR_FROM_TOP/$SRC_FILE_NAME"
   fi
   SRC_FILE_FULL_NAME="$PWD_WIN/$SRC_FILE_NAME"

#   echo "EZ_CURRENT_ROOT=$EZ_CURRENT_ROOT"
#   echo "PWD_WIN=$PWD_WIN"
#   echo "TOP_ROOT=$TOP_ROOT"
#   echo "DIR_FROM_TOP=$DIR_FROM_TOP"
#   echo "SYNC_TOP=$SYNC_TOP"
#   echo "SYNC_FILE_FULL_NAME=$SYNC_FILE_FULL_NAME"
#   echo "SRC_FILE_FULL_NAME=$SRC_FILE_FULL_NAME"

   if [ ! -f "$SRC_FILE_NAME" ] ; then
      if [ -f "$SYNC_FILE_FULL_NAME" ] ; then
         src.ksh stabalize -c deduce "$SYNC_FILE_FULL_NAME" "$SRC_FILE_NAME" \
           $STABALIZE_ARGS
      else
         echo "ERROR: Failed to find file $SRC_FILE_NAME."
         EXIT_CODE=1
         continue
      fi
   fi

   if [ ! -f "$SYNC_FILE_FULL_NAME" ] ; then
      echo "ERROR: Failed to find patch file $SYNC_FILE_FULL_NAME."
      EXIT_CODE=1
      continue
   fi
   if [ "$SYNC_FILE_FULL_NAME" = "$SRC_FILE_FULL_NAME" ] ; then
      echo "ERROR: Source and target are the same file."
      EXIT_CODE=1
      continue
   fi
   src.ksh stabalize -diffs "$SYNC_FILE_FULL_NAME" "$SRC_FILE_FULL_NAME" \
           $STABALIZE_ARGS
done

EZSrcCleanup
exit $EXIT_CODE
